/*
  History:
  Date       	Who				What
  Jan_30_2004  	jpsousa			Created public version
 */
package messaging;

import java.io.IOException;

import util.*;

/**
 * Base class for all point to point connectors in Aura.
 */

public abstract class AbstractPtoPConnector
						implements IConnector
{
//private
	// sends messages over the socket
	private 	ISender				_sender;

	// for queueing messages
	private 	IMessageQueue		_incomingQueue;
	private 	IMessageQueue		_outgoingQueue;

	// listener class, manages server socket,
	// incoming client sockets, message readers, etc
	private 	ThreadedListener	_listener;

	// for processing read messages into receive() method
	private 	MessageProcessorThread	_messageProcessor;

	// for sending messages from the outgoing queue
	private		MessageSenderThread     _messageSender;

	String _remoteHost = null;
	String _localHost = null;
	int _remotePort;
	int _localPort;

	protected final void setEndPoints(
								String remoteHost,
								int remotePort,
								String localHost,
								int localPort,
								String listenerName)
					throws java.io.IOException
	{
		_remoteHost = remoteHost;
		_localHost = localHost;
		_remotePort = remotePort;
		_localPort = localPort;

		_incomingQueue = new MessageQueue();
		_outgoingQueue = new MessageQueue();

		_sender = new SocketSender(	_remoteHost,_remotePort);

		_listener = new ThreadedListener(	_localHost,
											_localPort,
											listenerName,
											_incomingQueue);


		_messageProcessor = new MessageProcessorThread(
										listenerName,
									_incomingQueue,
									this);

		_messageSender = new MessageSenderThread( listenerName,
													_outgoingQueue,
												  this);
	}

	protected final String getRemoteHost()
	{
		return _remoteHost;
	}

	protected final int getRemotePort()
	{
		return _remotePort;
	}

	protected final String getLocalHost()
	{
		return _localHost;
	}

	protected final int getLocalPort()
	{
		return _localPort;
	}

	public final void send(String message)
	{
		if (_remoteHost != null)
		{
			try
			{
				MessageWrapper wrapper = new MessageWrapper();
				wrapper._message = message;
				wrapper._sender = _sender;
				_outgoingQueue.queueMessage(wrapper);
			}
			catch ( java.lang.InterruptedException interrupted)
			{
				Assert.report(interrupted);
			}
		}
	}

/*
	private void sendToDefault(String strMessage)
			throws java.io.IOException
	{
		System.out.println("Sending message ---> \n" + strMessage);
		_sender.sendMessage(strMessage);
	}
*/
	/**
	 * Called if a previous send attempt fails.
	 * Concrete classes must implement this method
	 * to react to the send failure appropriately.
	 * @param message that couldn't be sent
	 * @param exception
	 */
	public abstract void errorSending(String message,
										String remoteHost,
										int remotePort,
										IOException exception);

	/**
	 * Called after the connector receives a message
	 * from the remote host. Concrete classes must
	 * implement this method to parse the message
	 * and dispatch it appropriately
	 * @param message received from remote host
	 */
	public abstract void receive(String message);
}
