package messaging;
import util.*;

/**
*	This is a thread class for processing incmoing messages.
*   Reads messages of a message queue, and dispatches those
*   messages to methods in callbacks.
*	@see edu.gcmu.aura.connector.IMessageQueue
*/

class MessageProcessorThread extends Thread
{
/**
*	Queue to read from.
*/
	protected IMessageQueue 		_queue;

/**
*   Dispatcher to use.
*/
	protected IConnector  _receiver;

	protected boolean			m_bStopRunning = false;
	protected String	_connectorName;
/**
*	Initialize with a message queue and receiver.
*/
	public MessageProcessorThread (	String connectorName,
									IMessageQueue queue,
									IConnector receiver)
	{

		super( "MesssageProcessor-"+connectorName);
		_connectorName = 	connectorName;
		_queue = queue;
		_receiver = receiver;
		this.start();
	}

	public void run()
	{
		System.out.println("(Info) MessageProcessorThread-"
							+ _connectorName + " : starting");
		while ( ! m_bStopRunning  )
		{
			try
			{
				MessageWrapper wrapper = (MessageWrapper ) _queue.getNextMessage();
				String message = wrapper._message;
				try
				{
					if ( message != null )
					{
						_receiver.receive(message);
					}
					else
					{
						System.out.println("MessageProcessor :: run : idled for one cycle.");
//						System.out.println(_queue.getClass().getName());
						AuraSleep.sleep(50);
					}
				}
				catch ( java.lang.RuntimeException runtime)
				{
					runtime.printStackTrace();

				}
			}
			catch ( java.lang.InterruptedException interrupted)
			{
				m_bStopRunning = true;
				System.out.println("MessageProcessor :: run : interrupted exception received.  Aborting run loop.");
			}
			catch ( java.lang.RuntimeException runtime)
			{
				Assert.report(runtime);
			}
			if ( this.isInterrupted() )
				m_bStopRunning = true;
		}
	}
}