package messaging;

import util.AuraSleep;

/**
*	This is a thread class for processing incmoing messages.
*   Reads messages of a message queue, and dispatches those
*   messages to methods in callbacks.
*	@see edu.gcmu.aura.connector.IMessageQueue
*/

class MessageSenderThread extends Thread
{
/**
*	Queue to read from.
*/
	protected IMessageQueue 	_queue;

	protected boolean			m_bStopRunning = false;

	protected IConnector 	_errorHandler;
	protected String	_connectorName;
/**
*	Initialize with a message queue and receiver.
*/
	public MessageSenderThread (  String connectorName,
								  IMessageQueue queue,
								  IConnector errorHandler)
	{
		super("MessageSender-"+ connectorName);
		_connectorName = connectorName;
		_queue = queue;
		_errorHandler = errorHandler;
		this.start();
	}

	public void run()
	{
		System.out.println("(Info) MessageSenderThread-" +
							_connectorName + " : starting");

		while ( ! m_bStopRunning  )
		{
			try
			{

				MessageWrapper wrapper = (MessageWrapper ) _queue.getNextMessage();
				String message = wrapper._message;
				ISender sender = wrapper._sender;

				try
				{
					if ( message != null )
					{
						sender.sendMessage(message);
					}
					else
					{
						AuraSleep.sleep(50);
					}
				}
				catch ( java.io.IOException ioException )
				{
					_errorHandler.errorSending( message,
												sender.getRemoteHost(),
												sender.getRemotePort(),
												ioException);
				}
				catch ( java.lang.RuntimeException runtimeException )
				{
					runtimeException.printStackTrace();
					_errorHandler.errorSending( message,
							sender.getRemoteHost(),
							sender.getRemotePort(),
							new java.io.IOException(runtimeException.getMessage()));
				}
			}
			catch ( java.lang.InterruptedException interrupted)
			{
				m_bStopRunning = true;
				System.out.println("MessageSender :: run : interrupted exception received.  Aborting run loop.");
			}
			if ( this.isInterrupted() )
				m_bStopRunning = true;
		}
	}
}