package messaging;

/*
	This class implements the ISender interface
	using plain sockets and the Aura Wire protocol.
	See interfaces:
		ISender,
*/




import java.io.*;
import java.net.*;

import util.AuraSleep;


//public
class SocketSender
						implements ISender

{
	protected String 			m_strHost = null;
	protected int    			m_iPort = -1;

	protected String			m_strURL = null;

	protected Socket			m_socket = null;

	protected java.io.Writer 	m_writer = null;
	protected java.io.Reader	m_reader = null;

	protected int				_endpointCount;

	public SocketSender(	String host,
							int port)
					//	throws java.io.IOException
	{
		this.m_strHost = host;
		this.m_iPort = port;
		_endpointCount = 1;

/*
		m_socket = null;
		m_socket  = new Socket(m_strHost, m_iPort);
		initStreams();
*/
	}


	public SocketSender(	String strURL)
					//	throws java.io.IOException
	{

		m_strURL = strURL;
		_endpointCount = 1;
/*
		java.net.URL url = new java.net.URL(strURL);

		m_strHost = url.getHost();
		m_iPort = url.getPort();

		m_socket = null;
		m_socket  = new Socket(m_strHost, m_iPort);

		initStreams();
*/
	}

	public synchronized int addEndPoint()
	{
		_endpointCount ++;
		return _endpointCount;
	}

	public synchronized int removeEndPoint()
	{
		_endpointCount--;
		if ( _endpointCount < 0 )
			throw new IllegalStateException("Usage count below zero: " + _endpointCount);
		if ( _endpointCount == 0 )
		{
			try
			{
				disconnect();
			}
			catch (java.io.IOException ioExc )
			{}
		}
		return _endpointCount;
	}


	protected void getHostPortFromURL()
						throws java.io.IOException
	{
		java.net.URL url = new java.net.URL(m_strURL);

		m_strHost = url.getHost();
		m_iPort = url.getPort();
	}

	public void connect()
						throws java.io.IOException
	{
		if ( m_strHost == null )
			getHostPortFromURL();

		m_socket = null;
		m_socket  = new Socket(m_strHost, m_iPort);
		initStreams();
	}

	public void disconnect()
						throws java.io.IOException
	{
		if (  m_socket != null )
		{
			m_socket.close();
		}

		m_socket = null;
		m_reader = null;
		m_writer = null;
	}


	protected void initStreams()
						throws java.io.IOException
	{
		m_writer = new OutputStreamWriter(m_socket.getOutputStream());
		m_reader = new InputStreamReader(m_socket.getInputStream());
	}


	public void sendMessage(String strXmlMessage)
						throws java.io.IOException
	{
		if ( m_socket == null || (! m_socket.isConnected() ))
		{
			connect();
		}
		String strAuraMessage  = SocketProtocolMessage.createMessage(strXmlMessage);
		try
		{
			m_writer.write(strAuraMessage, 0, strAuraMessage.length());
			m_writer.flush();
		}
		catch (java.io.IOException ioExc)
		{
			ioExc.printStackTrace();
			AuraSleep.sleep(1000);
			this.connect();
			m_writer.write(strAuraMessage, 0, strAuraMessage.length());
			m_writer.flush();
		}
		return;
	}

	public String toString()
	{
		String strRet = new String(m_strHost + ":" + m_iPort);
		return strRet;
	}

	public String getRemoteHost()
	{
		return m_strHost;
	}

	public int getRemotePort()
	{
		return m_iPort;
	}


}