/*
  File: ThreadedListener.java

  History:
  Date       	Who				What
  May_19_2003  	poladian		Created public version
*/

package messaging;
import java.net.*;


/**
 *  This listener spawns a new thread upon each incoming client socket.
 *
 *  It is well suited for listening on relatively few sockets
 *  that are open for long periods of times.
 *  The communication between Prism and EM fits this pattern.
 *  Also, the connectors for the suppliers fit this pattern too.

 *  This listener derives from class java.lang.Thread, and accepts
 *  incoming sockets in its thread.  Once accepted, sockets are
 *  given to a newly created thred, an instance of SocketReader.
 *
 *  The processing of incoming messages in sockets is done
 *  in the run() method of SocketReader object.
 *
 * @see edu.cmu.aura.connector.PooledListener
 * @see edu.cmu.aura.connector.IMessageQueue
 *
 **/

class ThreadedListener extends Thread

{
	protected ServerSocket 			m_serverSocket;
	protected IMessageQueue		m_queue;
	protected boolean				m_bStopRunning = false;
	protected String				m_strName;

	protected ThreadedListener (	int port,
									IMessageQueue queue,
									String strName)
						throws java.io.IOException
	{
		m_queue = queue;
		m_strName = strName;
		m_serverSocket = new ServerSocket(port);
		initializeMembers();
		this.start();
	}

	ThreadedListener (	String strAddress,
								int port,
								String strName,
								IMessageQueue queue)
							throws java.io.IOException
	{
		m_queue = queue;
		m_strName = strName;
		InetAddress [] addresses = InetAddress.getAllByName(strAddress);
		if ( addresses != null && addresses.length > 0 )
		{
			m_serverSocket = new ServerSocket(  port,
												0,
												addresses[0]);
		}
		else
			m_serverSocket = new ServerSocket(port);

		initializeMembers();
		this.start();
	}

	protected void initializeMembers()
	{
	}

	public void interrupt()
	{
		m_bStopRunning = true;
	}

	/**  overrides the run() method in java.lang.Thread
	*
	*   starts accepting client sockets
	*   after accepting a socket, passes to a new instance of SocketReader classs
	**/
	public void run()
	{
		System.out.println("@@@@ ThreadedListener *" + m_strName + "* starting at address "
				+ m_serverSocket.getInetAddress().getHostAddress()
				+ ":" + m_serverSocket.getLocalPort());

		while ( !m_bStopRunning )
		{
			Socket client = null;
			try
			{
				client = m_serverSocket.accept();
				System.out.println("@@@@ ThreadedListener *" + m_strName + "* accepted socket from "
						+ client.getInetAddress().getHostAddress()
						+ ":" + client.getPort());

				NonYieldingSocketReaderThread socProc =
							new NonYieldingSocketReaderThread (m_strName,
												client,
												this.m_queue);

			}
			catch (java.io.IOException ioExc)
			{
				ioExc.printStackTrace();
			}
		}
	}

  /**
   *  Returns the listening address of this listener, in a url format.
   *  Typically, this is a string of the form: machine:port
   *  @return  listening address
   **/
	String getListenerAddress()
	{

		String strHost = m_serverSocket.getInetAddress().getHostAddress();
		int iPort = m_serverSocket.getLocalPort();
		String retVal = new String(strHost + ":" + iPort);
		return retVal;
	}
}
